<?php
    require_once('inc/common.php');

    /**
     * Generate HTML for a column header to use standard sorting on a list page
     *
     * Rather than try to explain everything, let me show examples:
     * makeSortHeader('Type Of Animal','Species');
     * If the current page isn't sorted by this already (or has a decending sort), this returns:
     * <a href="index.php?[most existing get variables]&Sort=asc&Order=Species>Type Of Animal</a>
     *
     * @param string $title Actual plain-text title of header.
     * @param string $gets (null) What to use as the GET value if this is selected
     * @return string
     */
    function makeSortHeader($title, $gets = null)
    {
        if (empty($gets)) {
            $gets = $title;
        }

        $preGETs = '';
        foreach ($_GET as $key=>$value) {
            if ($key == 'Order' && $value == $gets) {
                // Mark this column because it is the one being sorted
                $title = '['.$title.']';
            }
            elseif ($key != 'Sort' && $key != 'Order') {
                $preGETs .= "$key=$value&";
            }
        }
        $preGETs = substr($preGETs, 0,strlen($preGETs)-1);
        if (isset($gets)) {
            if (isset($_GET['Sort']) && isset($_GET['Order']) && $_GET['Order'] == $gets) {
                if ($_GET['Sort']=='asc')
                    $sort = 'desc';
                else
                    $sort = 'asc';

                return "<a href=\"index.php?$preGETs&Sort=$sort&Order=$gets\">$title&nbsp;<img src=\"images/arrow-$sort.png\"></a>";
            } else {
                return "<a href=\"index.php?$preGETs&Sort=asc&Order=$gets\">$title&nbsp;<img src=\"images/arrow-asc.png\"></a>";
            }
        } else {
            return $title;
        }
    }
?>