<?php

    //Constants for notification events.
    define('N_TICKETCREATED', 'N_TICKETCREATED');
    define('N_TICKETASSIGNED', 'N_TICKETASSIGNED');
    define('N_SUPPORTLOGUPDATED', 'N_SUPPORTLOGUPDATED');
    define('N_TICKETCOMPLETED', 'N_TICKETCOMPLETED');

    //Notification function.  $ID refers to the ID of the support ticket or the support log item, depending on the event.
    function notify($NotificationType, $ID) {
        global $settings;  //the jabber settings are required, so this variable is set to the global "settings" variable.
        switch ($NotificationType) {
            case N_TICKETCREATED:
                //When a ticket is created, send a notification to the requestor.
                $support = DB_DataObject::factory('support');
                $support->get($ID);

                if (!$support->Employee) {
                    return false;
                } else {
                    $recipient = $support->getLink("Employee", "Employees");
                }
                $emailSubject = "Support Ticket #" . $support->ID . " Opened";
                break;
            case N_TICKETASSIGNED:
                //When a ticket is assigned to a technician (or reassigned), send notification to the technician
                $support = DB_DataObject::factory('support');
                $support->get($ID);

                if (!$support->Employee) {
                    return false;
                } else {
                    $recipient = $support->getLink("Technician", "Employees");
                }
                $emailSubject = "Support Ticket #" . $support->ID . " (" . $support->Title . ") Assigned";
                break;
            case N_SUPPORTLOGUPDATED:
                //When a support log item is created, send a notification to the requestor.
                $supportLog = DB_DataObject::factory('supportLog');
                $supportLog->get($ID);
                $support = $supportLog->getLink("Issue", "Support");
                if (!$support->Employee) {
                    return false;
                } else {
                    $recipient = $support->getLink("Employee", "Employees");
                }
                $emailSubject = "Support Ticket #" . $support->ID . " Updated";
                break;
            case N_TICKETCOMPLETED:
                //When a ticket is closed, send a notification to the requestor.
                $support = DB_DataObject::factory('support');
                $support->get($ID);

                if (!$support->Employee) {
                    return false;
                } else {
                    $recipient = $support->getLink("Employee", "Employees");
                }
                $emailSubject = "Support Ticket #" . $support->ID . " Closed";
                break;
            default:
                exit;
                break;
        }

        if( $recipient->PreferredNotification == 'none' ) {
            return true;
        }

        $template = file_get_contents('inc/templates/' . $NotificationType . '.' . $recipient->PreferredNotification . '.tpl');

        //Place DataObjects into an array so that they can be accessed by a variable.
        $DOs['Support'] = $support;
        $DOs['SupportLog'] = $supportLog;
        $DOs['Recipient'] = $recipient;

        //List of static variables to search for.  As many can go here as needed.
        $templateReplacements = array(
            'SERVER' => $_SERVER["SERVER_NAME"],
            'TIME' => strftime($settings['DateFormat'] . " " . $settings['TimeFormat'])
        );

        //replace non-dataobject variables in the template.
        foreach($templateReplacements as $key => $data) {
            $template = str_replace('`' . $key . '`', $data, $template );
        }

        //replace any variables in the templates that include periods with the corresponding DataObject properties.
        while (ereg('\`([A-Za-z1-9\._]+)\`', $template, $tVar)) {
            //if there are two periods in the template variable, it will attempt to link the property
            //with the corresponding DataObject and then match the property within that linked DO.
            if (ereg('(.+)\.(.+)\.(.+)', $tVar[1], $tVars)) {
                if(!is_object($DOs[$tVars[1]]->$tVars[2]) && $DOs[$tVars[1]]->$tVars[2]) {
                    $DOs[$tVars[1]]->$tVars[2] = $DOs[$tVars[1]]->getLink($tVars[2]);
                }
                $template = str_replace($tVar[0], $DOs[$tVars[1]]->$tVars[2]->$tVars[3], $template );
            } elseif (ereg('(.+)\.(.+)', $tVar[1], $tVars)) {
                $template = str_replace($tVar[0], $DOs[$tVars[1]]->$tVars[2], $template );
            } else {
                $template = str_replace($tVar[0], 'ERROR', $template );
            }
        }

        //Send the notification using the preferred notification method defined by the user.
        switch ($recipient->PreferredNotification) {
            case "email":
                mail($recipient->EmailAddress, $emailSubject, $template, "From: ". $GLOBALS['settings']['Notifications']['emailFrom'] . "\r\n");
                break;
            case "jabber":
                require_once("inc/class.jabber.php");
                $j = new Jabber;
                $j->server = $settings['Notifications']['jabberServer'];
                $j->port = $settings['Notifications']['jabberPort'];
                $j->username = $settings['Notifications']['jabberUser'];
                $j->password = $settings['Notifications']['jabberPassword'];
                $j->resource = "OpenIT";
                $j->enable_logging = false;
                $j->Connect() or die('<b>Jabber Failure:</b> Connection Failed in <b>' .$_SERVER[PATH_TRANSLATED] . '</b>');
                $j->SendAuth() or die ('<b>Jabber Failure:</b> Login Failed in <b>' .$_SERVER[PATH_TRANSLATED] . '</b>');
                $j->SendPresence();
                $m = $j->SendMessage($recipient->JabberAccount,
                    "chat",
                    NULL,
                    array(
                        "body" => $template
                        )
                    );
                break;
        }
    }
?>