<?php
    chdir(dirname(__FILE__).'/../');
    require_once('inc/common.php');
    PEAR::setErrorHandling();
?>
<html>
    <head>
        <title>OpenIT <?=VERSION ?> Setup Check</title>
        <link rel="stylesheet" type="text/css" href="../css/main.css" />
    </head>
    <body>
        <div id="title">
            <h1>OpenIT <?=VERSION ?> Setup Check</h1>
        </div>
        <div id="toc">
            &nbsp;<a href="check.php">Re-check</a>
        </div>
        <div id="content">
        <ol>
<?php

    /**
     * The following are defined in DB/DataObject.php
     *
     * define('DB_DATAOBJECT_INT',  1);  // does not require ''
     * define('DB_DATAOBJECT_STR',  2);  // requires ''
     * define('DB_DATAOBJECT_DATE', 4);  // is date #TODO
     * define('DB_DATAOBJECT_TIME', 8);  // is time #TODO
     * define('DB_DATAOBJECT_BOOL', 16); // is boolean #TODO
     * define('DB_DATAOBJECT_TXT',  32); // is long text #TODO
     * define('DB_DATAOBJECT_BLOB', 64); // is blob type
     * define('DB_DATAOBJECT_NOTNULL', 128); // not null col.
     * define('DB_DATAOBJECT_MYSQLTIMESTAMP', 256); // mysql timestamps (ignored by update/insert)
     */

    // String to DataObject Constant
    function str2do_const($type) {
        global $db;

        switch (strtoupper($type)) {

            case 'INT':
            case 'INT2':    // postgres
            case 'INT4':    // postgres
            case 'INT8':    // postgres
            case 'SERIAL4': // postgres
            case 'SERIAL8': // postgres
            case 'INTEGER':
            case 'TINYINT':
            case 'SMALLINT':
            case 'MEDIUMINT':
            case 'BIGINT':
                $tempType = DB_DATAOBJECT_INT;
                if (strlen($type) == 1) {
                    $tempType +=  DB_DATAOBJECT_BOOL;
                }
                return $tempType;

            case 'REAL':
            case 'DOUBLE':
            case 'FLOAT':
            case 'FLOAT8': // double precision (postgres)
            case 'DECIMAL':
            case 'NUMERIC':
            case 'NUMBER':
                return DB_DATAOBJECT_INT; // should really by FLOAT!!! / MONEY...

            case 'YEAR':
                return DB_DATAOBJECT_INT;

            case 'BIT':
            case 'BOOL':
            case 'BOOLEAN':
                $tempType = DB_DATAOBJECT_BOOL;
                // postgres needs to quote '0'
                if ($db->phptype == 'pgsql') {
                    $tempType +=  DB_DATAOBJECT_STR;
                }
                return $tempType;

            case 'STRING':
            case 'CHAR':
            case 'VARCHAR':
            case 'VARCHAR2':
            case 'TINYTEXT':
            case 'ENUM':
            case 'SET':         // not really but oh well
            case 'TIMESTAMPTZ': // postgres
            case 'BPCHAR':      // postgres
            case 'INTERVAL':    // postgres (eg. '12 days')
            case 'CIDR':        // postgres IP net spec
            case 'INET':        // postgres IP
            case 'MACADDR':     // postgress network Mac address.
                return DB_DATAOBJECT_STR;

            case 'TEXT':
            case 'MEDIUMTEXT':
            case 'LONGTEXT':
                return DB_DATAOBJECT_STR + DB_DATAOBJECT_TXT;

            case 'DATE':
                return DB_DATAOBJECT_STR + DB_DATAOBJECT_DATE;

            case 'TIME':
                return DB_DATAOBJECT_STR + DB_DATAOBJECT_TIME;

            case 'DATETIME':
                return DB_DATAOBJECT_STR + DB_DATAOBJECT_DATE + DB_DATAOBJECT_TIME;

            case 'TIMESTAMP': // do other databases use this???
                return ($db->phptype == 'mysql') ?
                    DB_DATAOBJECT_MYSQLTIMESTAMP :
                    DB_DATAOBJECT_STR + DB_DATAOBJECT_DATE + DB_DATAOBJECT_TIME;

            case 'TINYBLOB':
            case 'BLOB':       /// these should really be ignored!!!???
            case 'MEDIUMBLOB':
            case 'LONGBLOB':
            case 'BYTEA':   // postgres blob support..
                return DB_DATAOBJECT_STR + DB_DATAOBJECT_BLOB;
           }
    }

    // DataObject Constant to String
    function do_const2str($type) {
        $TextType = '';

        if((DB_DATAOBJECT_INT & $type) == DB_DATAOBJECT_INT)
            $TextType .= 'Integer ';
        elseif((DB_DATAOBJECT_STR & $type) == DB_DATAOBJECT_STR)
            $TextType .= 'String ';

        if((DB_DATAOBJECT_DATE & $type)  == DB_DATAOBJECT_DATE)
            $TextType .= 'Date ';
        if((DB_DATAOBJECT_TIME & $type) == DB_DATAOBJECT_TIME)
            $TextType .= 'Time ';

        if((DB_DATAOBJECT_BOOL & $type) == DB_DATAOBJECT_BOOL)
            $TextType .= 'Boolean ';

        if((DB_DATAOBJECT_TXT & $type) == DB_DATAOBJECT_TXT)
            $TextType .= 'Text ';

        if((DB_DATAOBJECT_BLOB & $type) == DB_DATAOBJECT_BLOB)
            $TextType .= 'Blob ';

        if((DB_DATAOBJECT_MYSQLTIMESTAMP) & $type == DB_DATAOBJECT_MYSQLTIMESTAMP)
            $TextType .= 'MySQL Timestamp ';

        if((DB_DATAOBJECT_NOTNULL & $type) == DB_DATAOBJECT_NOTNULL)
            $TextType .= '<i>Not Null</i>';

        return $TextType;
    }

    // --MAIN CHECK LOOP-- \\

    $INISchema = parse_ini_file('inc/schema/openit.ini', true);
    //$Keys = parse_ini_file('inc/schema/openit.links.ini', true);

    foreach($INISchema as $Table => $INIFields)
    {
        if((substr($Table, -5) != '_keys') && substr($Table, -4) != '_seq') {

            echo '<li>Checking <i>'.$Table.'</i>... ';
            $ok = true;

            $DBSchema = $db->tableInfo($Table);
            if(DB::isError($DBSchema)) {
                $ok = false;
                echo "<b>Missing!</b> <br>Please create table with:<ul>";
                foreach($INIFields as $INIFieldName => $INIFieldType)
                {
                    echo '<li>'.$INIFieldName.': <code>'.
                        do_const2str($INIFieldType)."</code></li>\n";
                }
            }
            else {
                // Create this array for more easily comparing the INI to
                // the DB later.
                $DBFieldList = array();

                // Compare DB to INI
                foreach($DBSchema as $DBField)
                {
                    // 1st - See if DB Field exists in INI Schema
                    if(!array_key_exists($DBField['name'], $INIFields)) {
                        if($ok)
                            echo '<ul>';

                        echo '<li>The field, <code>'.$DBField['name'].
                            '</code>, should <b>not exist</b>.</li>';
                        $ok = false;
                    }
                    else {
                        // 2nd - Build a list of DB tables for later use
                        $DBFieldList[$DBField['name']] = 1;

                        // 3rd - Convert DB::tableInfo type to DO constant
                        $DBFieldType = str2do_const($DBField['type']);

                        // 4th - Compare DB Field Type to INI Field Type
                        if(($INIFields[$DBField['name']] & $DBFieldType) != $DBFieldType) {
                            if($ok)
                                echo '<ul>';

                            echo '<li>'.$DBField['name'].' is a <code>'.
                                $DBField['type'].'</code> and should be '.
                                'a <code>'.do_const2str($INIFields[$DBField['name']]).
                                '</code></li>';
                            $ok = false;
                        }
                    }
                }

                // Compare INI to DB
                foreach($INIFields as $INIFieldName => $INIFieldType)
                {
                    // 1st - See if INI Field exists in DB Schema
                    if(!array_key_exists($INIFieldName, $DBFieldList)) {
                        if($ok)
                            echo '<ul>';

                        $ok = false;
                        // 2nd Output INI Field to create
                        echo '<li>The field, <code>'.$INIFieldName.'</code> of '.
                            'type <code>'.do_const2str($INIFieldType).
                            '</code>, is <b>missing.</b> ';
                    }

                }
            }
            if($ok)
                echo "OK</li>\n";
            else
                echo "</ul></li>\n<br>\n";
        }

        // TODO: Insert code to test __keys information
    }
    echo "</ol>\n";
?>
    </body>
</html>