<?php

/* Note that database creation sql must be located in tools/setup.sql
 * This is an attempt to simplify setup.php scipt maintence.  Instead
 * of having to modify the script everytime a new database scheme is developed,
 * a simple dump from mysqldump or phpmyadmin should suffice.  Currently it
 * only supports CREATE TABLE statements. All other sql statements will create unexpecetd
 * results.  Also the statements MUST be terminated with semicolons (standard
 * MySQL procedure).  I plan on including support of INSERT etc... statements in
 * the future to exapand the functionality of this script to support database
 * backups, upgrades, and consistency checks but for now it will have to do.
 * -- jwc */

if (!file_exists('../inc/settings.ini')) {
    pHead("OpenIT Setup Error");
    echo "<h1>Error: inc/settings.ini does not exist.  Please create.</h1>";
    pTail(1);
 } elseif (get_magic_quotes_gpc() == 1) {
    pHead("OpenIT Setup Warning");
    echo "<h1>Warning: Magic Quotes should be disabled.</h1>\n",
         "<p>Putting quotes and apostrophes into OpenIT will produce unexpected\n",
         "results with Magic Quotes turned on.  Visit\n",
         "<a href=\"http://php.net/get-magic-quotes-gpc\">http://php.net/get-magic-quotes-gpc</a>\n",
         "for more information.</p>\n";
    pTail();
} elseif (isset($_POST['doit']) && $_POST['doit'] == "yes") {
    chdir(dirname(__FILE__).'/../');

    if(!($sqlfile = fopen('tools/setup.sql', 'rb'))) {
        pHead("OpenIT Setup Error");
        echo "<h1>Could not open the file tools/setup.sql.txt for reading!</h1>";
        pTail(1);
    }

    require_once('inc/common.php');
    pHead("OpenIT Database Configuration");
    echo "<p>Setting up database</p>\n <ol>\n";
    flush();

    $tables = array();
    while(!feof($sqlfile)) {
        $l = trim(fgets($sqlfile, 1024));
        // CREATE TABLE specifies the beginning of our record
        if(preg_match('/^CREATE TABLE (\w+).*/i', $l, $m)) {
            /* slurp up everything until we see the end of this CREATE TABLE
             * sql statment.  For now we only want database structure dumps
             * generated by mysqldump (or possibly phpMyAdmin -- although I
             * don't use it or have an installation of it to test. */
            $tables[] = $m[1];
            $query = '';

            do {
                if($query)
                    $query .= ' ';
                $query .= $l;
            } while(($l = trim(fgets($sqlfile, 1024))) &&
                !feof($sqlfile) && !preg_match('/(.*\)).*;/', $l, $m));

            $query .= " $m[1]";

            echo "<li>Setting up table ", $tables[count($tables) - 1], "...</li>\n";
            flush();
            $res = $db->query($query);
            if(PEAR::isError($res)) {
                echo "<li>Error: ", $db->getMessage(), "</li>\n",
                    "<li>Rolling back the database...</li>\n";
                rollback_db($db, $tables);
            }
            //echo "<pre>query is $query\n</pre>\n\n";
        }
    }
    $extraQuery = array();
    $extraQuery[] = "INSERT INTO `Groups`
        ( `GroupID` , `Name` , `ComputersPerm` , `PrintersPerm` , `PeripheralsPerm` , `SoftwarePerm` , `EmployeesPerm` , `ReportsPerm` , `SupportPerm` , `KBPerm` , `LookupsPerm` , `NewsPerm` , `GroupPerm` , `OutOfOfficePerm` , `PolicyPerm` , `Flag` )
        VALUES ('', 'Guests', '0', '0', '0', '0', '2', '0', '1', '2', '0', '1', '0', '1', '0', '0');";
    $extraQuery[] = "INSERT INTO `Groups`
        ( `GroupID` , `Name` , `ComputersPerm` , `PrintersPerm` , `PeripheralsPerm` , `SoftwarePerm` , `EmployeesPerm` , `ReportsPerm` , `SupportPerm` , `KBPerm` , `LookupsPerm` , `NewsPerm` , `GroupPerm` , `OutOfOfficePerm` , `PolicyPerm` , `Flag` )
        VALUES ('', 'Administrators', '4', '4', '4', '4', '4', '4', '4', '4', '4', '4', '4', '4', '4', '0');";
    $extraQuery[] = "INSERT INTO Departments (Description) VALUES ('Information Technology');";
    $extraQuery[] = "INSERT INTO `Employees`
        ( `EmployeeID` , `Contractor` , `ITTechnician` , `FirstName` , `LastName` , `MiddleInitial` , `Department` , `Campus` , `WorkPhone` , `HomePhone` , `MobilePhone` , `Fax` , `EmailAddress` , `Notes` , `HomeStreetAddress` , `HomePOBox` , `HomeCity` , `HomeState` , `HomeZip` , `Password` , `GroupID` , `Language` , `JabberAccount` , `PreferredNotification` , `Flag` )
        VALUES ('', '0', '0', 'Default', 'Administrator', NULL , 1 , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , NULL , SHA1( '' ) , '2', NULL , NULL , 'none', '0'
);";
    foreach($extraQuery as $query) {
        flush();
        $res = $db->query($query);
        if(PEAR::isError($res)) {
            echo "<li>Error: ", $db->getMessage(), "</li>\n",
                "<li>Rolling back the database...</li>\n";
            rollback_db($db, $tables);
        }
    }
?>
            </ol>
            <h1>Complete</h1>
            <p>Please go to your OpenIT <a href="../index.php">home page</a> and
                <a href="../login.php">sign on</a> as the <i>Default
                Administrator</i> to begin using OpenIT.
            </p>
<?php
        pTail();
}

pHead("OpenIT Database Setup");
?>
            <form action="setup.php" method="post">
            <input type="hidden" name="doit" value="yes" />
            <input type="submit" value="Install Schema" />
            </form>
<?php
pTail();

// Drops existing tables, and exits.
function rollback_db($db, $tables)
{
    foreach (array_reverse($tables) as $table) {
        echo "<li>Removing table $table</li>\n";
        flush();
        $db->query("DROP TABLE $table");
    }
    pTail(1);
}

// by creating these functions we eliminate the need to worry about
// ob_start() and friends, I ran into this prob initially.
function pHead($title = "OpenIT Database Setup")
{
?>
<html>
 <head>
  <title><?= $title ?></title>
  <link rel="stylesheet" type="text/css" href="../css/main.css" />
 </head>
 <body>
  <div id="title">
   <h1><?= $title ?></h1>
  </div>
  <div id="content">
<?
}

// Close HTML tags and exit
function pTail($exit = 0)
{
?>
  </div>
 </body>
</html>
<?php
    exit($exit);
}
?>